from dotenv import load_dotenv
from os import environ
from os.path import dirname, basename, isfile, join
import glob


class Provider:
    """
    Provider instance
    """
    def __init__(self, name: str, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)
        self.name: str = name

    def __repr__(self):
        return f'Provider({self.__dict__})'


class Providers:
    def __init__(self):
        pass


class EasyPay:
    """
    Create an instance of EasyPay
    """
    def __init__(self, **kwargs):
        load_dotenv()
        self.provider = Providers()
        for k, v in kwargs.items():
            setattr(self, k, v)
        if 'provider' not in self.__dict__ and 'providers' not in self.__dict__:
            raise ValueError('At least one provider is required')
        if 'providers' in self.__dict__:
            for provider in self.__dict__['providers']:
                self.configure_provider(provider)

    def configure_provider(self, provider: str | Provider, **kwargs):
        setattr(
            self.provider,
            provider.name if isinstance(provider, Provider) else provider,
            Provider(provider, **kwargs) if isinstance(provider, str) else provider
        )

    async def create_invoice(self, amount: int | float, currency: str = 'USD', provider: str | Provider = None):
        invoice = Invoice(self.provider, amount, currency)
        return await invoice.create(provider)

    def __repr__(self):
        return f'EasyPay({self.__dict__})'


class Invoice:
    """
    Invoice class, use it to create an invoice
    Args:
        amount: int
    Attributes:
        amount: int
        status: str
        identifier: str
    """
    def __init__(self, providers: Providers, amount: int | float, currency: str = 'USDT'):
        self.providers = providers
        self.currency = currency
        self.amount = amount
        self.status = 'creating'
        self.identifier = None
        self.pay_info = None
        self.invoice = None

    async def create(self, provider: str | Provider, run_check: bool = False):
        modules = glob.glob(join(dirname(__file__) + '/providers', "*.py"))
        __all__ = [basename(f)[:-3] for f in modules if isfile(f) and not f.endswith('__init__.py')]

        if ((isinstance(provider, str) and provider not in __all__) or
                (isinstance(provider, Provider) and provider.name not in __all__)):
            raise ValueError(
                f'Provider {provider.name if isinstance(provider, Provider) else provider} is not supported'
            )

        module = __import__(f'pyeasypay.core.providers.{provider.name if isinstance(provider, Provider) else provider}',
                            globals(), locals(), ['Invoice'], 0)
        try:
            self.invoice = module.Invoice(
                self.providers.__dict__[provider.name if isinstance(provider, Provider) else provider],
                self, self.amount
            )
        except KeyError:
            raise ValueError(
                f'Provider {provider.name if isinstance(provider, Provider) else provider} '
                f'was not added, please add it first'
            )
        await self.invoice.create()

        if run_check:
            print(NotImplementedError('run_check is not implemented yet, check invoice status manually'))

        return self

    async def check(self):
        if self.invoice is None:
            return ValueError('Invoice needs to be created first')
        return await self.invoice.check()

    def __repr__(self):
        return f'Invoice({self.__dict__})'



from aiocryptopay import AioCryptoPay, Networks
from aiocryptopay.exceptions import factory


class Invoice:
    def __init__(self, provider, invoice, amount):
        self.creds = provider
        self.invoice = invoice
        self.amount = amount

        if 'api_key' not in self.creds.__dict__.keys():
            raise ValueError(f'api_key is required for {self.creds.name} provider')

        if 'network' not in self.creds.__dict__.keys():
            self.creds.network = 'main'

        self.crypto = AioCryptoPay(token=self.creds.api_key,
                                   network=(Networks.MAIN_NET if self.creds.network == 'main' else Networks.TEST_NET)
                                   if self.creds.network else Networks.MAIN_NET)

    async def create(self):
        try:
            invoice = await self.crypto.create_invoice(asset=self.invoice.currency, amount=self.amount)
        except factory.CodeErrorFactory as e:
            await self.crypto.close()
            raise ValueError(f"Wasn't able to create invoice ({e.code}: {e.name})")
        self.invoice.pay_info = invoice.bot_invoice_url
        self.invoice.identifier = invoice.invoice_id
        self.invoice.status = invoice.status
        return invoice.bot_invoice_url

    async def check(self):
        invoice = await self.crypto.get_invoices(invoice_ids=self.invoice.identifier)
        if invoice.status != self.invoice.status:
            self.invoice.status = invoice.status
        if invoice.status == 'paid':
            await self.crypto.close()
        return self.invoice.status



from requests import post


class Invoice:
    def __init__(self, provider, invoice, amount):
        self.creds = provider
        self.invoice = invoice
        self.amount = amount

        if 'login' not in self.creds.__dict__.keys():
            raise ValueError(f'login is required for {self.creds.name} provider')

        if 'secret' not in self.creds.__dict__.keys():
            raise ValueError(f'secret is required for {self.creds.name} provider')

        if self.invoice.currency != 'RUB':
            raise ValueError(f'Only RUB currency is supported for {self.creds.name} provider')

    async def create(self):
        url = "https://api.crystalpay.io/v2/invoice/create/"
        payload = {
            "auth_login": self.creds.login,
            "auth_secret": self.creds.secret,
            "amount": self.amount,
            "type": 'purchase',
            "description": 'CrystalPay payment',
            "redirect_url": self.creds.redirect_url if 'redirect_url' in self.creds.__dict__.keys() else "https://nichind.dev",
            "callback_url": self.creds.callback_url if 'callback_url' in self.creds.__dict__.keys() else "https://nichind.dev",
            "lifetime": 1440
        }
        headers = {'Content-Type': 'application/json'}

        response = post(url, json=payload, headers=headers)
        data = response.json()

        if data.get("error"):
            raise ValueError(data.get("errors"))
        self.invoice.identifier = data.get("id")
        self.invoice.pay_info = data.get("url")
        return self.invoice.pay_info

    async def check(self):
        url = "https://api.crystalpay.io/v2/invoice/info/"
        payload = {
            "auth_login": self.creds.login,
            "auth_secret": self.creds.secret,
            "id": self.invoice.identifier
        }
        headers = {'Content-Type': 'application/json'}

        response = post(url, json=payload, headers=headers)
        data = response.json()

        self.invoice.status = 'paid' if data.get("state") == 'payed' else 'pending'
        return self.invoice.status



from hashlib import sha256
from uuid import uuid4
import aiohttp
import hashlib
from urllib.parse import urlencode
from asyncio import wait_for
from requests.exceptions import ConnectTimeout, ReadTimeout


class AsyncAaioAPI:
    """Originally written by https://github.com/wkillus/"""
    def __init__(self, API_KEY, SECRET_KEY, MERCHANT_ID):
        """
        Creates instance of one AAIO merchant API client

        Args:
            merchant_id: Merchant ID from https://aaio.so/cabinet
            secret: 1st secret key from https://aaio.so/cabinet
            api_key: API key from https://aaio.so/cabinet/api
        """
        self.API_KEY = API_KEY
        self.SECRET_KEY = SECRET_KEY
        self.MERCHANT_ID = MERCHANT_ID

    async def get_balance(self):
        """
        Creates a request for get balances of user
        See https://wiki.aaio.so/api/poluchenie-balansa

        Returns: Model from response JSON
        """

        URL = 'https://aaio.so/api/balance'

        headers = {
            'Accept': 'application/json',
            'X-Api-Key': self.API_KEY
        }

        try:
            async with aiohttp.ClientSession(headers=headers) as session:

                async with session.post(URL) as response:

                    if (response.status in [200, 400, 401]):
                        try:
                            response_json = await response.json()  # Парсинг результата
                        except:
                            return 'Не удалось пропарсить ответ'

                        if (response_json['type'] == 'success'):
                            return response_json
                        else:
                            return 'Ошибка: ' + response_json['message']  # Вывод ошибки
                    else:
                        return 'Response code: ' + str(response.status)  # Вывод неизвестного кода ответа

        except ConnectTimeout:
            return 'ConnectTimeout'  # Не хватило времени на подключение к сайту

        except ReadTimeout:
            return 'ReadTimeout'  # Не хватило времени на выполнение запроса

    async def create_payment(self, order_id,
                             amount, lang='ru',
                             currency='RUB', description=None):
        """
        Creates payment URL
        See https://wiki.aaio.so/priem-platezhei/sozdanie-zakaza for more detailed information

        Args:
            amount: Payment amount
            order_id: Your order id
            description: Payment description (Optional)
            currency: Payment currency
            language: Page language (Optional)

        Returns: Payment URL

        """

        merchant_id = self.MERCHANT_ID  # merchant id
        secret = self.SECRET_KEY  # secret key №1 from shop settings

        sign = f':'.join([
            str(merchant_id),
            str(amount),
            str(currency),
            str(secret),
            str(order_id)
        ])

        params = {
            'merchant_id': merchant_id,
            'amount': amount,
            'currency': currency,
            'order_id': order_id,
            'sign': hashlib.sha256(sign.encode('utf-8')).hexdigest(),
            'desc': description,
            'lang': lang
        }

        url_aaio = "https://aaio.so/merchant/pay?" + urlencode(params)

        return url_aaio

    async def get_payment_info(self, order_id):
        """
        Creates a request for get payment information
        See https://wiki.aaio.so/api/informaciya-o-zakaze

        Args:
            order_id: Your order ID

        Returns: Model from response JSON

        """

        URL = 'https://aaio.so/api/info-pay'

        params = {
            'merchant_id': self.MERCHANT_ID,
            'order_id': order_id
        }

        headers = {
            'Accept': 'application/json',
            'X-Api-Key': self.API_KEY

        }

        async with aiohttp.ClientSession(headers=headers) as session:
            async with session.post(URL, data=params) as response:
                response_json = await response.json()

                return response_json

    async def is_expired(self, order_id):
        """Check status payment (expired)"""

        response_json = await self.get_payment_info(order_id)

        return response_json['type'] == 'success' and response_json['status'] == 'expired'

    async def is_success(self, order_id):
        """Check status payment (success)"""

        response_json = await self.get_payment_info(order_id)

        return (response_json['type'] == 'success' and response_json['status'] == 'success') or \
            (response_json['type'] == 'success' and response_json['status'] == 'hold')


class Invoice:
    def __init__(self, provider, invoice, amount):
        self.creds = provider
        self.invoice = invoice
        self.amount = amount

        if 'api_key' not in self.creds.__dict__.keys():
            raise ValueError(f'api_key is required for {self.creds.provider} provider')

        if 'secret' not in self.creds.__dict__.keys():
            raise ValueError(f'secret is required for {self.creds.provider} provider')

    def create_signature(self):
        signature_string = f"{self.creds.api_key}:{self.amount}:{self.invoice.currency}:{self.creds.secret}:{self.invoice.identifier}"
        signature = sha256(signature_string.encode('utf-8')).hexdigest()
        return signature

    async def create(self):
        client = AsyncAaioAPI(self.creds.api_key, self.creds.secret, self.creds.api_key)

        self.invoice.identifier = str(uuid4())
        lang = self.creds.language if 'language' in self.creds.__dict__.keys() else 'en'

        signature = self.create_signature()
        self.invoice.pay_info = await client.create_payment(self.invoice.identifier, self.amount, lang, self.invoice.currency, 'AAIO Payment')
        return self.invoice.pay_info

    async def check(self):
        client = AsyncAaioAPI(self.creds.api_key, self.creds.secret, self.creds.api_key)

        expired = await wait_for(client.is_expired(self.invoice.identifier), timeout=10)
        success = await wait_for(client.is_success(self.invoice.identifier), timeout=10)

        if expired:
            self.invoice.status = 'expired'
            return 'expired'
        elif success:
            self.invoice.status = 'paid'
            return 'paid'
        else:
            self.invoice.status = 'pending'
            return 'pending'



from setuptools import setup, find_packages


def readme():
    with open('README.md', 'r') as f:
        return f.read()


setup(
    name='pyeasypay',
    version='0.2.4',
    author='nichind',
    author_email='nichinddev@gmail.com',
    description='Make money from your Python projects the easy way.',
    long_description=readme(),
    long_description_content_type='text/markdown',
    url='https://github.com/nichind/pyeasypay',
    packages=find_packages(),
    install_requires=[
        'aiohttp',
        'python-dotenv',
        'aiocryptopay',
        'requests',
        'sqlalchemy'
    ],
    classifiers=[
        'Programming Language :: Python :: 3.11',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent'
    ],
    python_requires='>=3.8'
)
